const project_name = "avon-landing";

// In this file change only project_name
const project = {
  text_domain: "" || project_name,
  packageName: "" || project_name,
  url: "http://localhost/" + project_name,

  uploads_dir: {
    files: "../../uploads/**",
    filesDest: "../../uploads/**",
  },

  scss: {
    src: "assets/scss/main.scss",
    dest: "dist/",
    watch: "assets/scss/**/*.scss",
  },
  js: {
    fileName: "main",
    src: "assets/js/main.js",
    dest: "dist/",
    watch: "assets/js/**/*.js",
  },
  icon_font: {
    font_name: "icon-font",
    src: "assets/font-icons/*.svg",
    dest: "assets/fonts/icons/",
    class: "icon",
    font_path: "../assets/fonts/icons/",
    path: "assets/scss/utilities/_icons-config.scss",
    targetPath: "../../scss/utilities/_icons.scss",
    formats: ["woff2", "woff", "ttf"],
  },
  php: {
    watch: "**/*.php",
  },
  settings: {
    browsers_autoprefixes: [
      "last 3 version",
      "> 1%",
      "ie >= 9",
      "ie_mob >= 10",
      "ff >= 30",
      "chrome >= 34",
      "safari >= 7",
      "opera >= 23",
      "ios >= 7",
      "android >= 4",
      "bb >= 10",
    ],
  },
};

const { parallel, series, dest, watch, src } = require("gulp"),
  babel = require("gulp-babel"),
  browserify = require("browserify"),
  buffer = require("vinyl-buffer"),
  source = require("vinyl-source-stream"),
  imagemin = require("gulp-imagemin"),
  pngcrush = require("imagemin-pngcrush"),
  sass = require("gulp-sass")(require("sass")),
  minifycss = require("gulp-uglifycss"),
  autoprefixer = require("gulp-autoprefixer"),
  mmq = require("gulp-merge-media-queries"),
  uglify = require("gulp-uglify"),
  rename = require("gulp-rename"),
  lineec = require("gulp-line-ending-corrector"),
  filter = require("gulp-filter"),
  sourcemaps = require("gulp-sourcemaps"),
  notify = require("gulp-notify"),
  browserSync = require("browser-sync").create(),
  wpPot = require("gulp-wp-pot"),
  sort = require("gulp-sort"),
  iconfont = require("gulp-iconfont"),
  iconfontCss = require("gulp-iconfont-css");

// Generate font from svg
function fontIcons() {
  return src([project.icon_font.src])
    .pipe(
      iconfontCss({
        fontName: project.icon_font.font_name,
        cssClass: project.icon_font.class,
        path: project.icon_font.path,
        targetPath: project.icon_font.targetPath,
        fontPath: project.icon_font.font_path,
      })
    )
    .pipe(
      iconfont({
        fontName: project.icon_font.font_name,
        prependUnicode: true,
        formats: project.icon_font.formats,
        normalize: true,
        centerHorizontally: true,
      })
    )
    .pipe(dest(project.icon_font.dest));
}

// Browser sync and local server
function browser_sync() {
  return browserSync.init({
    proxy: project.url,
    open: true,
    injectChanges: true,
  });
}

// Compailing SCSS
function styles() {
  let taskName = this.process.title;

  let sassConfig = {
    errLogToConsole: true,
    outputStyle: "expanded",
    precision: 10,
    includePaths: ["node_modules"],
  };
  if (taskName == "gulp build") sassConfig.outputStyle = "compressed";

  let stream = src(project.scss.src)
    .pipe(sourcemaps.init({ loadMaps: true }))
    .pipe(sass(sassConfig))
    .on("error", console.error.bind(console))
    .pipe(autoprefixer(project.settings.browsers_autoprefixes))
    .pipe(lineec())
    .pipe(filter("**/*.css"))
    .pipe(mmq({ log: true }))
    .pipe(rename({ suffix: ".min" }))
    .pipe(taskName == "gulp build" ? minifycss() : sourcemaps.write("."))
    .pipe(lineec())
    .pipe(dest(project.scss.dest))
    .pipe(filter("**/*.css"))
    .pipe(browserSync.stream())
    .pipe(
      notify({
        message: `TASK: "Site style" Completed! | ${taskName}`,
        onLast: true,
      })
    );
  return stream;
}

// Compiling site JS
function siteJS() {
  let taskName = this.process.title;
  let babaelCompact = taskName == "gulp build" ? true : false;
  let stream = browserify(project.js.src, { allowEmpty: true })
    .bundle()
    .pipe(source("main.js"))
    .pipe(buffer())
    .pipe(babel({ presets: ["@babel/env"], compact: babaelCompact }));

  if (taskName == "gulp build") stream.pipe(uglify());

  stream
    .pipe(browserSync.stream())
    .pipe(dest(project.js.dest))
    .pipe(
      notify({
        message: `TASK: "Site JS" Completed! | ${taskName}`,
        onLast: true,
      })
    );
  return stream;
}

// Generate pot file from theme
function translate() {
  return src(project.php.watch)
    .pipe(sort())
    .pipe(
      wpPot({
        domain: project.text_domain,
        package: project.packageName,
      })
    )
    .pipe(dest("languages/"))
    .pipe(notify({ message: 'TASK: "translate" Completed!', onLast: true }));
}

// Optimization image
function imageOptimization() {
  "use strict";

  return src(project.uploads_dir.files, { base: project.uploads_dir.files })
    .pipe(
      imagemin({
        interlaced: true,
        progressive: true,
        optimizationLevel: 7,
        svgoPlugins: [{ removeViewBox: false }],
        use: [pngcrush()],
      })
    )
    .pipe(dest(project.uploads_dir.filesDest));
}

// Watch JS and SCSS
function watchFiles() {
  watch(project.scss.watch, styles);
  watch(project.js.watch, siteJS);
}

// Tasks
exports.styles = styles; // gulp styles - build css
exports.siteJS = siteJS; // gulp siteJS - build custom JS
exports.translate = translate; // gulp translate - generate theme POT file
exports.images = imageOptimization; // gulp images - optimization uploaded assets

exports.fontIcons = series(fontIcons, styles); // gulp fontIcons - make icons from svg

// Main tasks
exports.default = series(parallel(browser_sync, watchFiles)); // gulp - default task with browser sync
exports.watch = watchFiles; // gulp watch - watch files without browser sync
exports.build = series(parallel(styles, siteJS, translate)); // gulp build - build site minify css and js
